
#include "triangulation.h"

void *
gen_assoc_data_n (int client, int entityid)
{
  if (client == 0) return 0;
  else             return (void *) (client * 10000 + entityid);
}

static void
got_the_right_data_n (void *ad, int client, int entityid)
{
  if (ad != gen_assoc_data_n (client, entityid)) {
    fprintf (stderr, "*** Mismatch for client %d, entity %d\n", client, entityid);
  }
}

void *
gen_assoc_data_tri (int client, int entityid)
{
  if (client == 0) return 0;
  else             return (void *) (client * 20000 - entityid);
}

static void
got_the_right_data_tri (void *ad, int client, int entityid)
{
  if (ad != gen_assoc_data_tri (client, entityid)) {
    fprintf (stderr, "*** Mismatch for client %d, entity %d\n", client, entityid);
  }
}

static void
test_assoc_data_n (triangulation_t t)
{
  int i, j, nc = 3;
  triangulation_node_p n;
  dict_iterator di = dict_get_iterator ();
  fprintf (stderr, "Testing node associated data\n");
  for (i = 1; i <= nc; i++) {
    fprintf (stderr, "Adding data for client %d\n", i);
    dict_reset_iterator (di);
    while (n = tri_next_node (t, di)) {
      tri_node_set_assoc_data (n, (triangulation_client_t) i, gen_assoc_data_n (i, tri_node_get_id (n)));
    }
    dict_reset_iterator (di);
    while (n = tri_next_node (t, di)) {
      for (j = 1; j <= i; j++) {
        fprintf (stderr, "Testing data for client %d\n", j);
        {
          triangulation_assoc_data_t ad = tri_node_get_assoc_data (n, (triangulation_client_t) j);
          got_the_right_data_n (ad, j, tri_node_get_id (n));
        }
      }
      for (j = i; j >= 0; j--) {
        fprintf (stderr, "Testing data for client %d\n", j);
        {
          triangulation_assoc_data_t ad = tri_node_get_assoc_data (n, (triangulation_client_t) j);
          got_the_right_data_n (ad, j, tri_node_get_id (n));
        }
      }
    }
  }
  dict_del_iterator (di);
}

static void
test_assoc_data_tri (triangulation_t t)
{
  int i, j, nc = 3;
  triangulation_triangle_p tri;
  dict_iterator di = dict_get_iterator ();
  fprintf (stderr, "Testing triangle associated data\n");
  for (i = 1; i <= nc; i++) {
    fprintf (stderr, "Adding data for client %d\n", i);
    dict_reset_iterator (di);
    while (tri = tri_next_triangle (t, di)) {
      tri_triangle_set_assoc_data (tri, (triangulation_client_t) i,
                                   gen_assoc_data_tri (i, tri_triangle_get_id (tri)));
    }
    dict_reset_iterator (di);
    while (tri = tri_next_triangle (t, di)) {
      for (j = 1; j <= i; j++) {
        fprintf (stderr, "Testing data for client %d\n", j);
        {
          triangulation_assoc_data_t ad = tri_triangle_get_assoc_data (tri, (triangulation_client_t) j);
          got_the_right_data_tri (ad, j, tri_triangle_get_id (tri));
        }
      }
      for (j = i; j >= 0; j--) {
        fprintf (stderr, "Testing data for client %d\n", j);
        {
          triangulation_assoc_data_t ad = tri_triangle_get_assoc_data (tri, (triangulation_client_t) j);
          got_the_right_data_tri (ad, j, tri_triangle_get_id (tri));
        }
      }
    }
  }
  dict_del_iterator (di);
}

static void
test_assoc_data (triangulation_t t)
{
  test_assoc_data_n (t);
  test_assoc_data_tri (t);
}

int
main ()
{
  FILE *stream;
  triangulation_t t;

  stream = fopen ("rosette-s1-f1.tg2", "r");
  if (stream == NULL) {
    fprintf (stderr, "could not open");
    exit (1);
  }

  t = tri_from_afro_file (stream);
  if (t == NULL) {
    fprintf (stderr, "error #%d\n", tri_status ());
    exit (1);
  }
  fclose (stream);

  test_assoc_data (t);

  tri_destroy_triangulation (t);

  return 0;
}
