#ifndef TRIANGULATION_H
#   define TRIANGULATION_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/stat.h>
#   include <unistd.h>
#   include <float.h>
#   if HAVE_STRING_H
#      include <string.h>
#   else
#      if HAVE_STRINGS_H
#         include <strings.h>
#      endif
#   endif
/* #define NDEBUG */
#include <assert.h>

#include "newP.h"
#include "ioP.h"
#include "dictP.h"

struct _triangulation_t;
struct _triangulation_triangle_t;
typedef struct _triangulation_triangle_t *triangulation_triangle_p;
struct _triangulation_node_t;
typedef struct _triangulation_node_t *triangulation_node_p;

typedef struct _triangulation_t *triangulation_t;

typedef enum  triangulation_status_t {
  TRIANGULATION_OK           = 0,
  TRIANGULATION_MALLOC       = 1,
  TRIANGULATION_BAD_TOTALS   = 2,
  TRIANGULATION_BAD_NODE     = 3,
  TRIANGULATION_BAD_TRIANGLE = 4
}             triangulation_status_t;


typedef void *triangulation_client_t;
typedef void *triangulation_assoc_data_t;

extern triangulation_t tri_from_targe2_file ( FILE *stream );
extern triangulation_t tri_from_afro_file ( FILE *stream );
extern triangulation_status_t tri_status ( void );
extern int tri_error_on_line ( void );
extern triangulation_node_p tri_next_node ( triangulation_t t, dict_iterator iterator );
extern triangulation_triangle_p tri_next_triangle ( triangulation_t t, dict_iterator iterator );
extern triangulation_node_p tri_find_node ( triangulation_t t, int node_id );
extern triangulation_triangle_p tri_find_triangle ( triangulation_t t, int id );
extern triangulation_assoc_data_t tri_node_get_assoc_data ( triangulation_node_p n, triangulation_client_t client );
extern triangulation_assoc_data_t tri_node_set_assoc_data ( triangulation_node_p n, triangulation_client_t client, triangulation_assoc_data_t newad );
extern triangulation_assoc_data_t tri_triangle_get_assoc_data ( triangulation_triangle_p tri, triangulation_client_t client );
extern triangulation_assoc_data_t tri_triangle_set_assoc_data ( triangulation_triangle_p tri, triangulation_client_t client, triangulation_assoc_data_t newad );
extern double tri_avg_edge_length ( triangulation_t t );
extern double tri_max_edge_length ( triangulation_t t );
extern double tri_max_edge_length_at_node ( triangulation_node_p n );
extern int tri_triangle_get_nodes ( triangulation_triangle_p tri, triangulation_node_p n[3] );
extern int tri_triangle_get_id ( triangulation_triangle_p tri );
extern int tri_triangle_get_group ( triangulation_triangle_p tri, int group_name_length, char *group, int *group_id );
extern int tri_node_get_xy ( triangulation_node_p n, double *x, double *y );
extern triangulation_triangle_p tri_node_next_incid_triangle ( triangulation_node_p n, triangulation_triangle_p tri );
extern int tri_node_num_incid_triangles ( triangulation_node_p n );
extern int tri_node_get_id ( triangulation_node_p n );
extern dict_dictionary tri_node_ring_nodes ( triangulation_node_p n, int ring_num );
extern void tri_destroy_triangulation ( triangulation_t t );
double
tri_triangle_get_area (triangulation_triangle_p tri);
triangulation_triangle_p 
tri_add_triangle (triangulation_t t,
                  int id,
                  triangulation_node_p n[3],
                  char *group
                  );
triangulation_node_p
tri_add_node (triangulation_t, int id, double x, double y);
int
tri_node_set_xy (triangulation_node_p n, double x, double y);
triangulation_t
tri_new_triangulation (void);
int
tri_to_targe2_file (triangulation_t t, FILE *stream);

#endif
